﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;

public class GameManager : MonoBehaviour {

	public GameObject[] columns;
	public GameObject soundEffectsObject;
	public GameObject paddle;
	public GameObject ballPrefab;
	public int firstBallCombo = 10;
	public int secondBallCombo = 20;
	public int combosPerExtraBall = 10;
	public int brickPoint = 10;
	public List<GameObject> ballsInPlay = new List<GameObject>();
	public List<GameObject> bricks = new List<GameObject>();
	public Text scoreText;
	public Text comboText;
	public Text ballsText;
	public Button playAgain;
	public int maxBalls = 5;

	private int combo = 1;
	private int score = 0;
	private SoundEffects soundEffects;
	private GameObject previousCollisionObject = null;
	private bool columnsReady = false;
	private int totalBalls;
	private int hitCount = 0;

	public void OnEnable() {
		BallScript.onBallCollideWithBrick += this.CollideWithBrick;
		BallScript.onBallCollideWithGround += this.CollideWithGround;
		BallScript.onBallCollideWithPaddle += this.CollideWithPaddle;
	}
	
	public void OnDisable() {
		BallScript.onBallCollideWithBrick -= this.CollideWithBrick;
		BallScript.onBallCollideWithGround -= this.CollideWithGround;
		BallScript.onBallCollideWithPaddle -= this.CollideWithPaddle;
	}


	public void CollideWithBrick(GameObject Ball, GameObject brick)
	{
		score += brickPoint * combo;
		updateScore();
		previousCollisionObject = brick;
		hitCount += 1;
		
		if (hitCount == bricks.Count)
		{
			hitCount = 0;
			foreach (GameObject brickItem in bricks)
			{
				brickItem.renderer.enabled = true;
				brickItem.collider.enabled = true;
			}
			StartCoroutine("enableColumns");
			soundEffects.PlayRespawnSound();
		}
	}

	public void CollideWithPaddle(GameObject BallScript, GameObject paddle)
	{
		if (previousCollisionObject != null)
		{
			combo += 1;
			updateComboCounter();
			if (combo == firstBallCombo && ballsInPlay.Count == 1 || combo == secondBallCombo && ballsInPlay.Count == 2)
			{
				spawnBall();
				launchBall();
				soundEffects.PlayExtraBallSound();
			}
			
		}
	}

	private void launchBall()
	{
		GameObject spawnPoint = paddle.transform.Find ("SpawnPoint").gameObject;
		Transform ball = spawnPoint.transform.Find("Ball");
		if (ball != null)
		{
			GameObject queuedBall = ball.gameObject;
			BallScript ballScript = queuedBall.GetComponent<BallScript>();
			ballScript.LaunchBall();
		}
	}
	
	private IEnumerator enableColumns() {
		for (int i=0; i < columns.Length; i++) {
			GameObject column = columns[i];
			column.SetActive(true);
			yield return new WaitForSeconds(.2f);
		}
		StopCoroutine("enableColumns");
		columnsReady = true;
	}

	void Awake() {
		playAgain.gameObject.SetActive(false);
		foreach (GameObject column in columns)
		{
			int brickCount = column.transform.childCount;
			for (int i = 0; i < brickCount; i++)
			{
				GameObject brick = column.transform.GetChild(i).gameObject;
				if (brick.tag.Equals("Brick"))
				{
					bricks.Add (brick);
				}
			}
			
		}

		soundEffects = soundEffectsObject.GetComponent<SoundEffects>();
		StartCoroutine("enableColumns");
		soundEffects.PlayRespawnSound();
		totalBalls = maxBalls;
	}

	public void CollideWithGround(GameObject ball, GameObject block)
	{
		previousCollisionObject = null;
		BallScript ballScript = ball.GetComponent<BallScript>();
		ballScript.Explode();
		combo = 1;
		totalBalls -= 1;
		updateBallCounter();
		if (totalBalls > 0) {
			spawnBall();
		} else {
			playAgain.gameObject.SetActive(true);
		}
	}	

	public void RestartGame()
	{
		playAgain.gameObject.SetActive(false);
		foreach (GameObject column in columns)
		{
			column.SetActive(false);
		}
		ballsInPlay.Clear();
		totalBalls = maxBalls;
		
		foreach (GameObject brick in bricks)
		{
			brick.renderer.enabled = true;
			brick.collider.enabled = true;
		}
		
		columnsReady = false;
		score = 0;
		combo = 1;
		hitCount = 0;
		totalBalls = maxBalls;
		columnsReady = false;
		updateScore();
		StartCoroutine("enableColumns");
		soundEffects.PlayRespawnSound();
		
	}

	private void updateBallCounter()
	{
		ballsText.text = "Balls: " + totalBalls;
	}

	private void updateComboCounter()
	{
		comboText.text = "Combo: x" + combo;
	}

	private void updateScore()
	{
		scoreText.text = "Score: " + score;
	}

	private void spawnBall()
	{
		GameObject spawnPoint = paddle.transform.Find ("SpawnPoint").gameObject;
		GameObject newBall = Instantiate(ballPrefab, spawnPoint.transform.position, Quaternion.identity) as GameObject;
		newBall.transform.localScale = Vector3.one;
		newBall.transform.parent = spawnPoint.transform;
		newBall.transform.name = "Ball";
		newBall.rigidbody.isKinematic = true;
		
		ballsInPlay.Add(newBall);
	}

	// Use this for initialization
	void Start () {
	
	}
	
	// Update is called once per frame
	void Update () {
		if (Input.GetKey(KeyCode.Space))
		{
			launchBall();
		}
		if (columnsReady == true && ballsInPlay.Count == 0)
		{
			spawnBall();
		}
	}
}
